/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

import java.awt.*;
import java.io.*;
import java.awt.event.*;

class PickCellFrame extends Frame
  implements ActionListener, WindowListener, ItemListener {  

  protected static final String
    OK = "OK",
    CANCEL = "Cancel";

  public static final int
    ROCK = 0,
    PLANT = 1;

  protected Font f = new Font ("Helvetica", 0, 12);

  protected World world;
  protected WorldFrame frame;
  protected UI ui;

  protected Button
    okButton, cancelButton;
    
  private Choice typeChoice;
  
  private Panel choicePanel;

  protected boolean
    rocks,
    plants,
    again;
      
  PickCellFrame(World world, UI ui, boolean rocks, boolean plants, boolean again)
  {
    this.ui = ui;
    this.world = world;
    this.frame = world.frame;
    this.rocks = rocks;
    this.plants = plants;
    this.again = again;
    
    if (plants) {
    choicePanel = new Panel();
    typeChoice = new Choice();
    typeChoice.setFont(f);
    typeChoice.addItem("Rocks");
    typeChoice.addItem("Fertile cells");
    typeChoice.addItemListener(this);
    choicePanel.add(typeChoice);
    }

    TextArea tA;
    if (plants && rocks)
	tA = new TextArea("Select rock cells and fertile cells." +
			  "\nClick on a selected cell to unselect it",
			  2, 35);
			  
    else if (rocks)
	tA = new TextArea("Select cells for rocks." +
			  "\nClick on a selected cell to unselect it.",
			  2, 35);

    else
	tA = new TextArea("Select fertile cells." +
			  "\nClick on a selected cell to unselect it.",
			  2, 30);
    
    Panel bPanel = new Panel();
    bPanel.setLayout (new FlowLayout());
    okButton = new Button(OK);
    okButton.addActionListener(this);
    bPanel.add(okButton);
    cancelButton = new Button(CANCEL);
    cancelButton.addActionListener(this);
    bPanel.add (cancelButton);

    // Put everything together
    setLayout (new BorderLayout());
    add("North", tA);
    if (plants && rocks)
      add ("Center", choicePanel);
    add ("South", bPanel);

    frame.initSelect();
    if (rocks)
      frame.select(Rock.COLOR, EditWorldFrame.ROCK);
    else
      frame.select(Cell.FERTILITY_COLOR, EditWorldFrame.FERTILE_CELL);
    
    addWindowListener (this);

    if (plants && rocks)
      setTitle("Place rocks & fertile areas");
    else if (plants)
      setTitle("Select fertile cells");
    else
      setTitle("Place rocks");
      
    pack();
  
    setVisible (true);
  } // constructor PickCellFrame

  public void windowClosed (WindowEvent event) {
  } // method windowClosed

  public void windowDeiconified (WindowEvent event) {
  } // method windowDeiconified

  public void windowIconified (WindowEvent event) {
  } // method windowIconified

  public void windowActivated (WindowEvent event) {
  } // method windowActivated

  public void windowDeactivated (WindowEvent event) {
  } // method windowDeactivated

  public void windowOpened (WindowEvent event) {
  } // method windowOpened

  public void windowClosing (WindowEvent event) {
    dispose();
  } // method windowClosing
  
  public void actionPerformed (ActionEvent event) {
      Object source = event.getSource();
      int c;
      
      if (source == okButton && frame.anySelectedCells()) {
	  
	  if (plants && frame.getNSelectedCells(true) < Params.getWorld(Params.NPLANTS))
	      ui.write("You must select at least " + Params.getWorld(Params.NPLANTS) +
		       " fertile cells!");
          else {
	  
  	    if (plants)
	        world.setFertileCells(frame.getSelectedCells(true));

  	    if (rocks)
	    for (c = 0; c < frame.getNSelectedCells(false); c++) {
		  new Rock(world, ui).addCell(frame.getSelectedCell(c, false));
	    }

	    frame.deinitSelect();
	    dispose();
	    world.createPlants(plants, true);
	    if (!rocks)
		world.createRocks();
            world.createCritters(again);
	    world.paint();
          }
	  
      } else {
	  frame.deinitSelect();
	  frame.repaint();
	  dispose();
      }
  } // method actionPerformed

  public void itemStateChanged (ItemEvent event) {
    int i = typeChoice.getSelectedIndex();
    if (i == ROCK) {
      frame.select(Rock.COLOR, EditWorldFrame.ROCK);
    } else {
      frame.select(Cell.FERTILITY_COLOR, EditWorldFrame.FERTILE_CELL);
    }
  } // method itemStateChanged

//  public Dimension getMinimumSize() {
//    return new Dimension (150, 100);
//  } // method getMinimumSize
  
//  public Dimension getPreferredSize() {
//    return getMinimumSize();
//  } // method getPreferredSize

} // class PickCellFrame
