/** 
 * SALSA 2
 * December 2000
 * Mike Gasser (gasser@indiana.edu)
*/

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

/**
* The user interface for the program, containing one TextArea
* where all sorts of messages appear, another TextArea for
* help only, two menus for Options and Windows, and Buttons
* for doing everything else.
*/

public class UI extends Frame 
  implements MouseListener, WindowListener, ItemListener, ActionListener {

  public static final double
    VERSION = 2.0;
    
  public static final String
    NAME = "SALSA";
    
  private static final int
    GENERATION = 0,
    MAX_VERBOSITY = 3,
    TEXT_AREA_WIDTH = 45,
    TEXT_AREA_HEIGHT = 8,
    HELP_TEXT_WIDTH = 35,
    HELP_TEXT_HEIGHT = 2;

  public static final String
    INIT_HELP_TEXT =
      "For help, select 'Show help' in the 'Options' menu.",
    HELP_INSTRUCTIONS =
      "For help with a window or window component,\n" +
      "place the cursor over it.",
    SEPARATOR = System.getProperty ("file.separator"),
    OS = System.getProperty ("os.name");

  private static final Font
    TEXT_FONT = new Font("Monospaced", Font.PLAIN, 12),
    BOLD = new Font("SansSerif", Font.BOLD, 12);
  
  // Various settable display parameters
  public static int
    cellSize = 6,
    animationStepPause = 50,
    withinStepPause = 100,
    stepPause = 0,
    verbosity = 0,
    nRunTimeSteps = 400;
    
  // Other display parameters
  public boolean
    animationEnabled = false,
    bestCritterSelect = false,
    critterSelect = false,
    showHelp = false;
  
  public World world = null;
  
  private Vector
    windows,
    windowTitles;

//  private PrintWriter transcript = null;
  
  private Critter
    bestCritter = null,
    selectedCritter = null;

  // Components
  private TextArea
    text,
    helpTA;
  
  private SalsaButton
    runButton,
    interruptButton,
    editButton,
//    thingButton,
    stepButton,
    reinitButton,
    quitButton;

  private Menu mw = new Menu ("Windows");
  
  private MenuItem
    newEnvMI = new MenuItem("New environment..."),
    loadWorldMI = new MenuItem("Load setup or environment..."),
    saveWorldMI = new MenuItem("Save environment..."),
    loadCritMI = new MenuItem("Load critters..."),
    saveCritMI = new MenuItem("Save critter..."),
    paramMI = new MenuItem("Set display parameters");
    
  private CheckboxMenuItem
    showStrengthMI = new CheckboxMenuItem("Show strength&age window", true),
    showLTMMI = new CheckboxMenuItem("Show LTM window", false),
    showSTMMI = new CheckboxMenuItem("Show STM window", false),
    showReinfMI = new CheckboxMenuItem("Show reinforcement window", false),
    RLMI = new CheckboxMenuItem("Critters change", true),
    dieMI = new CheckboxMenuItem("Critters die", false),
    raysMI = new CheckboxMenuItem("Show vision/sonar rays", false),
    animMI = new CheckboxMenuItem("Animate", false),
    transcriptMI = new CheckboxMenuItem ("Make transcript", false);
//    ckHelpMI = new CheckboxMenuItem ("Show help", false);
    
  public UI() {
    int i = 0;
    
    // Menus and menu items
    
    MenuBar mb = new MenuBar();
    Menu mf = new Menu("File");
    Menu mo = new Menu("Options");
    
    paramMI.addActionListener(this);
    loadWorldMI.addActionListener(this);
    saveWorldMI.addActionListener(this);
    loadCritMI.addActionListener(this);
    saveCritMI.addActionListener(this);
    newEnvMI.addActionListener(this);

    showLTMMI.addItemListener(this);
    showSTMMI.addItemListener(this);
    showStrengthMI.addItemListener(this);
    showReinfMI.addItemListener(this);

    RLMI.addItemListener(this);
    dieMI.addItemListener(this);
    raysMI.addItemListener(this);
    animMI.addItemListener(this);
    transcriptMI.addItemListener(this);
//    ckHelpMI.addItemListener(this);
    
    mf.add(newEnvMI);
    mf.add(loadWorldMI);
    mf.add(saveWorldMI);
    mf.add(loadCritMI);
    mf.add(saveCritMI);
    
    mo.add(paramMI);
    mo.add(RLMI);
    mo.add(dieMI);
    mo.add(raysMI);
    mo.add(animMI);
    mo.add(transcriptMI);
    mo.add(showLTMMI);
    mo.add(showSTMMI);
    mo.add(showStrengthMI);
    mo.add(showReinfMI);
//    mo.add(ckHelpMI);

    // Buttons and button panels
    
    Panel buttons = new Panel();
    buttons.setLayout (new BorderLayout());
        
    Panel northButtonPanel = new Panel();
    northButtonPanel.setLayout (new FlowLayout());
    Panel northButtons = new Panel();
    northButtonPanel.add (northButtons);    
    northButtons.setLayout (new FlowLayout());
    
    Panel southButtonPanel = new Panel();
    southButtonPanel.setLayout (new FlowLayout());
    Panel southButtons = new Panel();
    southButtonPanel.add (southButtons);
    southButtons.setLayout (new FlowLayout());

    editButton =
      new SalsaButton (this, this, "Edit world",
      "Click on this button to add or delete things\n" +
      "in the world window.");

    runButton =
      new SalsaButton (this, this, "Run",
      "Click on this button to run the things in\n"
      + "the world for a limited number of time steps\n"
      + "or until nothing else happens.");

    interruptButton =
      new SalsaButton (this, this, "Interrupt",
      "Click on this button to stop the world.");

    stepButton =
      new SalsaButton (this, this, "Step",
      "Click on this button to run the things in\n"
      + "the world for one time step.");
      
    reinitButton =
      new SalsaButton (this, this, "Reinit world",
      "Click on this button to reinitialize the world.");
      
    quitButton =
      new SalsaButton (this, this, "Quit",
      "Click on this button to quit SALSA.");
      
    northButtons.add(runButton);
    northButtons.add(stepButton);
    northButtons.add(interruptButton);
    
    southButtons.add(reinitButton);
    southButtons.add(editButton);
    southButtons.add(quitButton);

    buttons.add("North", northButtonPanel);
    buttons.add("South", southButtonPanel);
    
    windowTitles = new Vector();
    windows = new Vector();

    setLayout(new BorderLayout());

    mb.add(mf);
    mb.add(mo);
    mb.add(mw);
    setMenuBar(mb);

    text = new TextArea(TEXT_AREA_HEIGHT, TEXT_AREA_WIDTH);
    text.setEditable(false);
    text.setFont(TEXT_FONT);

    /*
    Panel helpPanel = new Panel();
    helpPanel.setLayout  (new FlowLayout());
    
    Label helpLabel = new Label ("HELP", Label.LEFT);
    helpLabel.setFont (BOLD);

    helpTA =
      new TextArea (INIT_HELP_TEXT,
        HELP_TEXT_HEIGHT, HELP_TEXT_WIDTH);
    helpTA.setEditable (false);

    helpPanel.add (helpLabel);
    helpPanel.add (helpTA);
    */
    
    add ("North", text);
//    add ("Center", helpPanel);
    add ("South", buttons);

    addWindowListener (this);

    setTitle(NAME);

    pack();

    setVisible (true);
    
    writeStartup();

    Params.set();
    Species.set();
    Environment.BASIC = new Environment(this);
    Params.read(this, getDataDirectory(), Params.FILE);
    world = new World(this);
//    createWorld();
//    write("User directory: " + System.getProperty("user.dir"));
  } // constructor UI
  
  public int getNRunTimeSteps() {
      return nRunTimeSteps;
  } // getNRunTimeSteps()
  
  /*
  private void createWorld() {
      world = new World(this);
      addWindow(world.frame);
      world.frame.setLocation(this.getSize().width, 0);
  } */ // createWorld()
    
  public void windowClosed (WindowEvent event) {
  } // method windowClosed

  public void windowDeiconified (WindowEvent event) {
  } // method windowDeiconified

  public void windowIconified (WindowEvent event) {
  } // method windowIconified

  public void windowActivated (WindowEvent event) {
  } // method windowActivated

  public void windowDeactivated (WindowEvent event) {
  } // method windowDeactivated

  public void windowOpened (WindowEvent event) {
  } // method windowOpened

  public void windowClosing (WindowEvent event) {
    System.exit(0);
  } // method windowClosing

  public void mouseClicked  (MouseEvent event) {
  } // method mouseClicked

  public void mousePressed (MouseEvent event) {
  } // method mousePressed
    
  public void mouseReleased (MouseEvent event) {
  } // method mouseReleased

  public void mouseEntered (MouseEvent event) {
    if (showHelp) {
      SalsaButton b = (SalsaButton)(event.getComponent());
	  helpWrite (b.helpText);
    }
  } // method mouseEntered

  public void mouseExited (MouseEvent event) {
    if (showHelp) {
      helpWrite (HELP_INSTRUCTIONS);
    }
  } // method mouseExited

  public void addWindow (Frame f) {
    String label = f.getTitle();
    MenuItem mi = new MenuItem (label);
    mi.addActionListener (this);
    windowTitles.addElement (label);
    windows.addElement (f);
    mw.add (mi);
  } // method addWindow
  
  public void removeWindow (Frame f) {
      if (f != null) {
	  
	  String label = f.getTitle();
	  MenuItem mi;
	  windowTitles.removeElement (label);
	  windows.removeElement (f);
	  // Remove the menu item for this window.
	  for (int i=0; i<mw.getItemCount(); i++) {
	      mi = mw.getItem (i);
	      if (mi.getLabel().equals (label))
		  mw.remove (mi);
	  }
	  f.dispose();
      }
  } // method removeWindow

  private Frame getWindow (int index) {
    return ((Frame)(windows.elementAt (index)));
  } // method getWindow
  
  private String makeFilename (String date) {
    return
      (NAME + "." + 
        ((date == null) ? Utils.dateString() : date));
  } // method makeFilename

  private String makeCompetFilename() {
    return Utils.removeExtension(Params.envFilename)
           + "_" + Params.getSpeciesNames() + ".trans";
  } // method makeCompetFilename()
  
  public static String makePathname(String fname, String dirname) {
    return (System.getProperty("user.dir")
            + SEPARATOR + dirname + SEPARATOR + fname);
  } // method makePathname
  
  public void selectCritter(Critter c) {
      selectedCritter = c;
  } // selectCritter(Critter)

  public PrintWriter openTranscript() {
    PrintWriter out = null;
    String fname = makeCompetFilename();

    try {
      write("Opening transcript file " + fname);
      out = new PrintWriter(new FileOutputStream(fname, true));

    } catch (IOException e) {
	System.err.println("UI: openTranscript: " + e + ".");
    }
    return (out);
  } // method openTranscript()
  
  public void actionPerformed (ActionEvent event) {
      Object source = event.getSource();
      
      // Deal with button presses
      if (source instanceof Button) {
	      
	  if (source.equals(quitButton)) {
	      System.exit(0);
	      
	  } else if (source.equals(stepButton)) {
	      animationEnabled = true;
	      world.step(-1);
	      animationEnabled = false;
	      
	  } else if (source.equals(reinitButton)) {
//	      if (world == null) {
//		  if (Params.read(this, getDataDirectory(), Params.FILE))
//		      createWorld();
//	      } else
	        world.reinit();
	      
	  } else if (source.equals(editButton)) {
	      EditWorldFrame ewf = new EditWorldFrame(world, this);

	  } else if (source.equals(runButton)) {
	      world.start();
	      
	  } else if (source.equals(interruptButton)) {
	      world.interrupt();

	  } else {
	      // Some unknown button
	      write("Don't know how to do this action!");
	  }
	  
      } else if (source instanceof MenuItem) {
      
        if (source.equals(paramMI)) {
	    ParamFrame tf = new ParamFrame(this);
	    
        } else if (source.equals(loadWorldMI)) {
          FileDialog fd = new FileDialog(this, "Select setup/environment file", 
					 FileDialog.LOAD);
          String filename;
          
	  fd.setVisible (true);
          filename = fd.getFile();
      
          if (filename != null) {
	      Params.envFilename = filename;
	      world.reinit(fd.getDirectory(), filename);
          }
	  
        } else if (source.equals(newEnvMI)) {
	    MakeEnv me = new MakeEnv(this, "Create new environment");

	} else if (source.equals(saveWorldMI)) {
	    FileDialog fd = new FileDialog(this, "Save environment file", FileDialog.SAVE);
	    fd.setVisible(true);
	    
	    world.write(fd.getFile(), fd.getDirectory());
	    
//	    FilenameDialog d =
//		new FilenameDialog(this, "Environment file name", "envX", false);
        
        } else if (source.equals(loadCritMI)) {
          FileDialog fd = new FileDialog(this, "Select critter file", FileDialog.LOAD);
          String filename;
          
	  fd.setVisible (true);
          filename = fd.getFile();
      
          if (filename != null) {
	      Species species = Params.readCritSpecies(this, filename, fd.getDirectory(), 10);
	      NumberDialog d = new NumberDialog(this, "Number of critters", "5", species);
	      
	      species.init();
	      world.createSpecies(Params.getNSpecies() - 1);
	      world.initFrames(true, true);
	      world.paint();
	      species.describe();
          }

	} else if (source.equals(saveCritMI)) {
//	    FileDialog fd = new FileDialog(this, "Save critter file", FileDialog.SAVE);
	    
	    String filename;
	    
	    FilenameDialog d =
		new FilenameDialog(this, "Select critter and enter name", "A", true);
        
        } else {
 	  String winname = event.getActionCommand();
          int windowI = windowTitles.indexOf (winname);
             
          if (windowI != -1) {
            getWindow (windowI).toFront();
          } else {
	      ;
          }
        }

      } else {
        write ("Don't know how to handle this sort of action!");
      }
  } // method actionPerformed

  public void itemStateChanged (ItemEvent event) {
     Object source = event.getItemSelectable();
     
     if (source.equals(showLTMMI)) {
	 world.setVisibleCritterFrame(World.LTM, showLTMMI.getState());
	 
     } else if (source.equals(showSTMMI)) {
	 world.setVisibleCritterFrame(World.STM, showSTMMI.getState());
	     
     } else if (source.equals(showStrengthMI)) {
	 world.setVisibleCritterFrame(World.STRENGTH, showStrengthMI.getState());
	     
     } else if (source.equals(showReinfMI)) {
	 world.setVisibleCritterFrame(World.REINFORCEMENT, showReinfMI.getState());

     /*
     if (source.equals (ckHelpMI)) {
       showHelp = ckHelpMI.getState();
       if (showHelp)
         helpWrite (HELP_INSTRUCTIONS);
       else
         helpWrite (INIT_HELP_TEXT);

     } else
     */
     /*
     if (source.equals (transcriptMI)) {

	 if (transcriptMI.getState() == true)
	 transcript = openTranscript();
	 else {
	     transcript.close();
	     transcript = null;
	 }
     } */
     
     } else if (source.equals(raysMI) && !raysMI.getState())
	 world.paint();
	 
  } // method itemStateChanged

  /** If verbosity >= minVb, then write s.
  */
  public void write (String s, int minVb) {
    if (verbosity >= minVb)
      text.append(nSpaces(minVb) + s + "\n");
//    if (transcript != null)
//      transcript.print (nSpaces (minVb) + s);
  } // method write
  
  /** Produce a string consisting of 2n spaces.
  */
  private String nSpaces(int n) {
      String s = "";
      for (int i = 0; i < n; i++) {
	  s += "  ";
      }
      return s;
  } // method nSpaces
  
  public void write(String s) {
  	write(s, 0);
  } // method write
  
  public void helpWrite(String s) {
  	helpTA.setText(s);
  } // method helpWrite

  public void writeStartup() {
    write("This is " + NAME + " (Version " + VERSION + ")");
//	   + "OS: " + OS + ".\n");
  } // writeStartup()
  
  /**
  * Show animation.
  */
  public boolean animate() {
      return (animationEnabled && animMI.getState());
  } // animate()
  
  /**
  * Do learning and ageing of Critters.
  */
  public boolean age() {
      return RLMI.getState();
  } // age()
  
  /**
  * Have Critters die, rather than get reincarnated, when
  * they run out of strength.   
  */
  public boolean die() {
      return dieMI.getState();
  } // die()
  
  public boolean showRays() {
      return (animationEnabled && raysMI.getState());
  } // showRays()
  
  public boolean makeTranscript() {
      return transcriptMI.getState();
  } // makeTranscript()
  
  public boolean showLTM() {
      return showLTMMI.getState();
  }
  
  public boolean showSTM() {
      return showSTMMI.getState();
  }
  
  public boolean showStrengths() {
      return showStrengthMI.getState();
  }
  
  public boolean showReinforcements() {
      return showReinfMI.getState();
  }
  
  /**
  * Should a Critter be selected in the World Frame?
  */
  public boolean critterToBeSelected() {
      return critterSelect;
  } // method critterToBeSelected
  
  /**
  * Set the highlighted Critter to be the given one.  If the critter
  * is already highlighed, unhighlight it.
  */
  public void selectHighlightCritter (Critter critter) {
      int canvasI;
      
      if (selectedCritter != null) {
          selectedCritter.unhighlight();
      }
      
      if (selectedCritter != critter) {
        selectedCritter = critter;
        critter.setHighlight();
        critterSelect = false;
      }
  } // method selectHighlightCritter

  public static String getDataDirectory() {
      return (System.getProperty ("user.dir") + UI.SEPARATOR + Params.DIRECTORY + UI.SEPARATOR);
  } // getDataDirectory()
  
} // class UI

class FilenameDialog extends Dialog implements ActionListener {

  protected static final String
    OK = "OK",
    CANCEL = "Cancel";

  protected Button
    okButton, cancelButton;

  String name;
  TextField tf;
  UI ui;
  World world;
  WorldFrame worldFrame;
  boolean select = false;

  FilenameDialog(UI ui, String title, String def, boolean select) {
    super(ui, title, !select);

    this.ui = ui;
    this.world = ui.world;
    this.select = select;
    if (select) {
        worldFrame = world.frame;
        worldFrame.initSelectCritter();
    }

    setLayout (new BorderLayout());

    tf = new TextField(def, 30);
    tf.addActionListener(this);
    
    Panel bPanel = new Panel();
    bPanel.setLayout(new FlowLayout());
    okButton = new Button(OK);
    okButton.addActionListener(this);
    bPanel.add(okButton);
    cancelButton = new Button(CANCEL);
    cancelButton.addActionListener(this);
    bPanel.add (cancelButton);

    add("North", tf);
    add("South", bPanel);

    pack();
    setVisible(true);
  } // constructor FilenameDialog(UI, String, String, boolean)

  public void actionPerformed(ActionEvent event) {
      Object source = event.getSource();
      int c;
      
      if (source.equals(okButton) || source.equals(tf)) {
	  if (select) {
	      if (!worldFrame.critterSelected())
  	        ui.write("You must first select a critter in the World window!");
	      else {
		dispose();
		FileDialog fd = new FileDialog(ui, "Save critter file", FileDialog.SAVE);
    	        fd.setVisible(true);
	        ui.write("Selected critter: " + worldFrame.getSelectedCritter().getName());
	    
	        world.write(fd.getFile(), fd.getDirectory(),
			    worldFrame.getSelectedCritter(), tf.getText());
//	        world.write(tf.getText(), worldFrame.getSelectedCritter());
	      }
	  
	  }  else {
//	      world.write(tf.getText());
	      dispose();
	  }

      } else
	  // "Cancel"
	  dispose();
  } // actionPerformed(ActionEvent)
	  
} // class FilenameDialog

class NumberDialog extends Dialog implements ActionListener {

  private static final String
    OK = "OK",
    CANCEL = "Cancel";

  private Button
    okButton, cancelButton;

  private String name;
  private TextField tf;
  private Species species;
  private UI ui;

  NumberDialog(UI ui, String title, String def, Species spec) {
    super(ui, title, true);

    this.ui = ui;
    this.species = spec;

    setLayout (new BorderLayout());

    tf = new TextField(def, 30);
    tf.addActionListener(this);
    Panel bPanel = new Panel();
    bPanel.setLayout(new FlowLayout());
    okButton = new Button(OK);
    okButton.addActionListener(this);
    bPanel.add(okButton);
    cancelButton = new Button(CANCEL);
    cancelButton.addActionListener(this);
    bPanel.add (cancelButton);

    add("North", tf);
    add("South", bPanel);

    pack();
    setVisible(true);
  } // constructor NumberDialog(UI, String, String, boolean)

  public void actionPerformed(ActionEvent event) {
      Object source = event.getSource();
      int c;
      
      if (source.equals(okButton) || source.equals(tf)) {
	  int n = Integer.parseInt(tf.getText());
	  if (n > 0) {
	      species.set(Params.N, n);
	      dispose();
	  }
	  else
	      ui.write("You must specify an integer!");

      } else if (source.equals(cancelButton))
	  // "Cancel"
	  dispose();
      else
	  ui.write("Hunh?");

  } // actionPerformed(ActionEvent)
  
} // class NumberDialog

