/** 
 * SALSA 2
 * July 2000
 * Mike Gasser (gasser@indiana.edu)
 */

/**
* This class is a place to keep general-purpose methods
* handy.
*/

import java.awt.Color;
import java.util.Date;
import java.text.SimpleDateFormat;

public class Utils {
    
  /**
  * Returns a String starting with s and containing
  * n characters.
  */
  public static String padString(String s, int n) {
	int i;
	String res;
	
	if (s.length() <= n) {
	    res = s;
	    // Add spaces
	    for (i = s.length(); i < n; i++) {
	        res += " ";
	    }
	} else {
	    res = s.substring (0, n);
	}
	
	return res;

  } // method padString
  
  public static String removeExtension(String s) {
      int index = s.lastIndexOf('.');
      if (index >= 0)
	  return s.substring(0, index);
      else
	  return s;
  } // removeExtension(String)
  
  public static void writeIntArray(int[] array, UI ui) {
      String s = "";
      for (int i = 0; i < array.length; i++)
	  s += array[i];
      
      ui.write("[" + s + "]");
  } // method writeIntArray(int[], UI)

  public static double gauss(double x, double sig) {
    return (Math.exp (- (x * x) / (2.0 * sig)));
  } // method gauss(double, double)

  /**
  * Returns the complement of the given color.
  */
  public static Color getComplement(Color c) {
       return (new Color (255 - c.getRed(),
			  255 - c.getGreen(),
			  255 - c.getBlue()));
  } // method getComplement
  
  // For the next 3, x is between 0 and 1.
  
  public static Color doubleToRed(double x) {
     int colorContrib = 255 - (int) (255.0 * x);
     return (new Color (255, colorContrib, colorContrib));
  }

  public static Color doubleToGreen(double x) {
     int colorContrib = 255 - (int) (255.0 * x);
     return (new Color (colorContrib, 255, colorContrib));
  }
  
  public static Color doubleToBlue(double x) {
     int colorContrib = 255 - (int) (255.0 * x);
     return (new Color (colorContrib, colorContrib, 255));
  }

  // Any negative or positive value.
  public static Color doubleToGreenRed(double x, double gain) {
      double tanhX = tanh(x, gain);
      boolean neg = x < 0.0;
      double absX = (neg ? -tanhX : tanhX);
      int colorContrib = 255 - (int) (255.0 * absX);
      if (neg)
	// red
	return (new Color(255, colorContrib, colorContrib));
      else
	// green
        return (new Color(colorContrib, 255, colorContrib));
  } // doubleToGreenRed(double)

  public static boolean getRandomBoolean() {
      return (Math.random() < .5);
  } // method getRandomBoolean
  
  public static double round(double x, int places) {
      double mult = Math.pow (10.0, (double)places);
      
      return (Math.round (x * mult) / mult);
  } // method round
    
  // It's Pacific time, but what the heck.
  public static String dateString() {
    SimpleDateFormat formatter =
      new SimpleDateFormat("yyMMdd.HHmm");
      
    return (formatter.format(new Date()));
  } // method dateString

  public static int chooseByDistribution(double[] distribution) {
      double
        value = Math.random(),
	cumProb = 0.0;
      int i;

      i = 0;
//      cumProb += distribution[0];

      while (value > cumProb && i < distribution.length) {
//	i++;
        cumProb += distribution[i];
	i++;
      }
      return i - 1;
  } // method chooseByDistribution(double[])

  /**
  * Just divides each double by the sum.
  */
  public static void scale(double[] vector) {
      double sum = 0.0;
      int i;
      for (i = 0; i < vector.length; i++)
	  sum += vector[i];

      if (sum != 0.0)
	  for (i = 0; i < vector.length; i++)
	      vector[i] /= sum;
  }
  
  public static double squash(double x, double gain) {
    return (1.0 / (1.0 + Math.exp (gain * ( - x))));
  } // method squash(double)
  
  public static double tanh(double x, double gain) {
      double y = Math.exp(gain * (- x));
      return ((1.0 - y) / (1.0 + y));
  } // method tanh(double, double)

  public static double booleanToDouble(boolean b) {
    if (b)
      return 1.0;
    else
      return 0.0;
  } // method booleanToDouble(boolean)
  
  public static int[] concatenateIntVectors(int[][] vectors) {
      int i, j, k = 0;
      int[] vec = new int[vectors.length * vectors[0].length];
      
      for (i = 0; i < vectors.length; i++) {
          for (j = 0; j < vectors[0].length; j++, k++) 
	      vec[k] = vectors[i][j];
      }
      return vec;
  } // method concatenateIntVectors(int[][])
  
  /**
  * Inserts an array of integers into a larger array of integers.
  */
  public static void insertVector(int[] obj, int[] dest, int start) {
      int i, j;
      for (i = start, j = 0; j < obj.length; i++, j++)
	  dest[i] = obj[j];
  } // method insertVector
  
  /**
   * Assumes the lengths of the vectors match.
   */
  public static double dotProduct(double[] x, double[] y) {
      double sum = 0.0;
	  
      for (int i = 0; i < x.length; i++)
	 sum += x[i] * y[i];

      return sum;
  } // method dotProduct(double[], double[])
  
  /**
  * x a double between 0 and 1
  */
  public static void double2Thermometer(double x, int[] thermo) {
      int n = thermo.length;
      double nDbl = (double)n;
      int i;
      for (i = 0; i < n; i++) {
          if (((i + 1) / nDbl) > x)
	      thermo[i] = 0;
	  else
	      thermo[i] = 1;
      }
  } // method double2Thermometer(double, int[])
  
  /**
  * x a double between 0 and 1.  resolution the number of possible different
  * values.   
  */
  public static double roundDouble(double x, int resolution) {
      double incr = 1.0 / resolution;
      double nRes = (double)resolution;
      int i = 0;
      while (i < resolution && ((i + 1) / nRes) <= x) {
          i++;
      }
	  
      return i * incr;
  } // method roundDouble(double, int)
  
    public static int roundDoubleToInt(double x, int resolution) {
      double nRes = (double)resolution;
      int i = 0;
      while (i < resolution && ((i + 1) / nRes) <= x) {
          i++;
      }
	  
      return i;
  } // method roundDouble(double, int)
  
  /**
  * x a double between 0 and 1.  start a position within thermo where
  * the representation of x should start.   
  */
  public static void double2Thermometer(double x, int[] thermo, int start, int length) {
      int n = thermo.length;
      int end = start + length;
      double nDbl = (double)length;
      int i, j;
      for (i = start, j = 0; i < end && i < n; i++, j++) {
          if (((j + 1) / nDbl) > x)
	      thermo[i] = 0;
	  else
	      thermo[i] = 1;
      }
  } // method double2Thermometer(double, int[])
  
  /**
  * Returns a double between 0 and 1 representing the proportion
  * of the distance between min and max where x is.   
  */
  public static double getRangeProportion(int min, int max, int x) {
      if (x < min)
	  x = min;
      else if (x > max)
	  x = max;
	  
      return ((double)(x - min)) / (max - min);
  } // method getRangeProportion(int, int, int)
  
  /**
  * Returns the square of 1 - x.
  */
  public static double reverseExaggerate(double x) {
      double rev = 1.0 - x;
      return rev * rev * rev;
  } // method reverseExaggerate(double)

} // class Utils
